@@ -31,7 +31,7 @@ import sys
 def mamba_install(prefix, specs, args, env, *_, **kwargs):
     # TODO: support all various ways this happens
     init_api_context()
-
+    api.Context().target_prefix = prefix
     # Including 'nodefaults' in the channels list disables the defaults
     channel_urls = [chan for chan in env.channels if chan != 'nodefaults']
 
@@ -39,7 +39,7 @@ def mamba_install(prefix, specs, args, env, *_, **kwargs):
         channel_urls.extend(context.channels)
     _channel_priority_map = prioritize_channels(channel_urls)
 
-    index = get_index(tuple(_channel_priority_map.keys()))
+    index = get_index(tuple(_channel_priority_map.keys()), prepend=False)
 
     channel_json = []
 
@@ -51,49 +51,46 @@ def mamba_install(prefix, specs, args, env, *_, **kwargs):
             # ignore non-loaded subdir if channel is != noarch
             continue
 
-        channel_json.append((chan, subdir.cache_path(), priority, subpriority))
+        channel_json.append((chan, subdir, priority, subpriority))
 
-    specs = [MatchSpec(s) for s in specs]
-    mamba_solve_specs = [s.conda_build_form() for s in specs]
-
-    print("\n\nLooking for: {}\n\n".format(mamba_solve_specs))
+    if not (context.quiet or context.json):
+        print("\n\nLooking for: {}\n\n".format(specs))
 
     solver_options = [(api.SOLVER_FLAG_ALLOW_DOWNGRADE, 1)]
 
     pool = api.Pool()
     repos = []
 
-    for channel, cache_file, priority, subpriority in channel_json:
-        repo = api.Repo(pool, str(channel), cache_file, channel.url(with_credentials=True))
+    for channel, subdir, priority, subpriority in channel_json:
+        repo = subdir.create_repo(pool)
         repo.set_priority(priority, subpriority)
         repos.append(repo)
 
     solver = api.Solver(pool, solver_options)
-    solver.add_jobs(mamba_solve_specs, api.SOLVER_INSTALL)
+    solver.add_jobs(specs, api.SOLVER_INSTALL)
     success = solver.solve()
     if not success:
         print(solver.problems_to_str())
+        exit(1)
 
     package_cache = api.MultiPackageCache(context.pkgs_dirs)
     transaction = api.Transaction(solver, package_cache)
+    if not (context.quiet or context.json):
+        transaction.print()
     mmb_specs, to_link, to_unlink = transaction.to_conda()
 
     specs_to_add = [MatchSpec(m) for m in mmb_specs[0]]
 
-    to_link_records, to_unlink_records = [], []
+    final_precs = IndexedSet()
 
-    final_precs = IndexedSet(PrefixData(prefix).iter_records())
-
-    def get_channel(c):
-        for _, chan in index:
-            if str(chan) == c:
-                return chan
+    lookup_dict = {}
+    for _, c in index:
+        lookup_dict[c.url(with_credentials=True)] = c
 
     for c, pkg, jsn_s in to_link:
-        sdir = get_channel(c)
+        sdir = lookup_dict[c]
         rec = to_package_record_from_subjson(sdir, pkg, jsn_s)
         final_precs.add(rec)
-        to_link_records.append(rec)
 
     unlink_precs, link_precs = diff_for_unlink_link_precs(prefix,
                                                           final_precs=IndexedSet(PrefixGraph(final_precs).graph),
@@ -117,7 +114,9 @@ def mamba_install(prefix, specs, args, env, *_, **kwargs):
 
 conda.install = mamba_install
 
+
+
 def main():
     from conda_env.cli.main import main
     sys.argv = sys.argv[0:1] + sys.argv[2:]
-    main()
+    return main()
